# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import json
import logging
import os

from flask import jsonify, send_file, abort
from flask_restplus import Resource, Namespace, reqparse

from langauge.core.service.main.database import mongo
from langauge.core.service.main.worker import celery

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

env = os.environ
"""
Provides all the task related information. 
"""
celery_task_ns = Namespace('celery', description='All task related information')


@celery_task_ns.route('/<task_id>', endpoint='task_status')
@celery_task_ns.param('task_id', 'The status identifier')
class TaskStatus(Resource):
    @celery_task_ns.doc('Provides status information of a particular task')
    def get(self, task_id):
        task = celery.AsyncResult(task_id)
        if task.state == 'PENDING':
            # job did not start yet
            response = {
                'state': task.state,
                'current': 0,
                'total': 1,
                'status': 'Pending...'
            }
        elif task.state != 'FAILURE':
            response = {
                'state': task.state,
                'current': task.info.get('current', 0),
                'total': task.info.get('total', 1),
                'status': task.info.get('status', '')
            }
            response['id'] = task_id
            if 'preview' in task.info:
                response['preview'] = task.info['preview']
            if 'time' in task.info:
                response['time'] = task.info['time']
        else:
            # something went wrong in the background job
            response = {
                'state': task.state,
                'current': 1,
                'total': 1,
                'status': str(task.info),  # this is the exception raised
            }
        return jsonify(response)


@celery_task_ns.route('/fileDownload/<task_id>')
@celery_task_ns.param('task_id', 'The task_id identifier')
class DownloadFile(Resource):
    @celery_task_ns.doc('Provides file related to a particular task')
    def get(self, task_id):
        outputs = env.get('OUTPUT_FOLDER')  # + task_id + '-00000-of-00001'
        file = ""
        for i in os.listdir(outputs):
            if os.path.isfile(os.path.join(outputs, i)) and task_id in i:
                file = os.path.join(outputs, i)
                break
        try:
            return send_file(file,
                             mimetype='text/csv',
                             attachment_filename='results.txt',
                             as_attachment=True)
        except:
            abort(404)



def task_already_running(channel):
    """
    Checks the number of tasks already runing. Returns true if its more than 4, otherwise false.
        
    :param channel: channel that submitted that task
    """
    i = celery.control.inspect()
    if i.active() is not None:
        for key, value in i.active().items():
            if len(value) >= 4:
                logger.log("there are already 2 task runing, Please submit after some time !!")
                return True
            # for task in value:
            #     if task.get('id') == channel:
            #         return True
        return False


parser = reqparse.RequestParser()
parser.add_argument('page_size', type=int,
                    help="number of records per page",
                    default=10, required=True)
parser.add_argument('page_num', type=int,
                    help="the page identifier",
                    default=1, required=True)


@celery_task_ns.route('/history')
@celery_task_ns.expect(parser)
class TaskHistory(Resource):
    @celery_task_ns.doc('Get all the task history')
    def get(self):
        args = parser.parse_args()
        # Calculate number of documents to skip
        skips = args.get('page_size') * (args.get('page_num') - 1)
        documents = mongo.db.celery_taskmeta.find()
        taskList = []
        for history in documents:
            if 'preview' in history.get('result'):
                continue
            history['id'] = history.pop('_id')
            result = json.loads(history.pop('result'))
            history['model'] = result.get('model', '')
            history['task'] = result.get('task', '')
            taskList.append(history)
        # return taskList
        return taskList[skips:skips+args.get('page_size')]


@celery_task_ns.route('/search/<task_id>')
@celery_task_ns.param('task_id', 'The task_id identifier')
class GetTask(Resource):
    @celery_task_ns.doc('Get the task related to the specific task id ')
    def get(self, task_id):
        from bson.json_util import dumps
        documents = mongo.db.celery_taskmeta.find(
            {'_id': {"$regex": task_id, "$options": "$i"}}
        )
        taskList = []
        documents = json.loads(dumps(documents))
        logger.info(documents)
        for document in documents:
            logger.info(document)
            document['id'] = document.pop('_id')
            taskList.append(document)
            logger.info(taskList)
        return taskList


@celery_task_ns.route('/history/total')
class GetTask(Resource):
    @celery_task_ns.doc('Get the total number of tasks')
    def get(self):
        documents = mongo.db.celery_taskmeta.find()
        count = 0
        for history in documents:
            if 'preview' in history.get('result'):
                continue
            count = count+1
        return count


# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import os

DEBUG = True

env = os.environ

HOST = env.get("HOST", "0.0.0.0")
PORT = env.get("PORT", "5000")

BASE_DIR = os.path.abspath(os.path.dirname(__file__))

UPLOAD_FOLDER = env.get("UPLOAD_FOLDER", os.path.dirname(__file__)+'/upload')
OUTPUT_FOLDER = env.get("OUTPUT_FOLDER", os.path.dirname(__file__)+'/output')
CONFIG_FOLDER = env.get("CONFIG_FOLDER", os.path.dirname(__file__)+'/config')

CELERY_BROKER_URL = env.get("CELERY_BROKER_URL", "redis://localhost:6379"),
CELERY_RESULT_BACKEND = env.get("CELERY_RESULT_BACKEND", "mongodb://root:root@localhost:27017/celery?authSource=admin")


# Enable protection against *Cross-site Request Forgery (CSRF)*
CSRF_ENABLED = True

# Use a secure, unique and absolutely secret key for
# signing the data.
CSRF_SESSION_KEY = "secret"

# Secret key for signing cookies
SECRET_KEY = "secret"

CORS_HEADERS = "Content-Type"

MONGO_URI = env.get("CELERY_RESULT_BACKEND", "mongodb://root:root@localhost:27017/celery?authSource=admin")


import os

import click
from controller.compose import Compose
from langauge.utils import cli_args
import dotenv

_ROOT = os.path.abspath(os.path.dirname(__file__))


@click.group(chain=True)
def cli():
    pass


# @cli.command()
# def ui():
#     subprocess.call("npm start", shell=True, cwd='ui/')
#     print("In UI")


@cli.command()
@cli_args.HOST
@cli_args.PORT
@cli_args.BACKEND
def ui(host, port, backend):
    dotenv_file = dotenv.find_dotenv(filename=os.path.join(_ROOT, 'ui', '.env'))
    dotenv.load_dotenv(dotenv_file)
    dotenv.set_key(dotenv_file, "HOST", host)
    dotenv.set_key(dotenv_file, "PORT", port)
    dotenv.set_key(dotenv_file, "REACT_APP_BASE_URL", 'http://'+backend)
    dc = Compose(files=[os.path.join(_ROOT, 'docker-compose.yml')])
    options = {
        "-d": True,
        "--no-color": True,
        "--quiet-pull": True,
        "--no-deps": True,
        "--force-recreate": True,
        "--always-recreate-deps": True,
        "--no-recreate": False,
        "--no-build": False,
        "--no-start": False,
        "--build": True,
        "--abort-on-container-exit": False,
        "--attach-dependencies": False,
        "--renew-anon-volumes": True,
        "--remove-orphans": True,
        "SERVICE": ["ui"],
        "--scale": ["service=1"]
    }
    dc.command('up', options)


@cli.command()
@cli_args.BACKENDPORT
def backend(backendport):
    dotenv_file = dotenv.find_dotenv(filename=os.path.join(_ROOT, '.env'))
    dotenv.load_dotenv(dotenv_file)
    dotenv.set_key(dotenv_file, "PORT", backendport)
    dc = Compose(files=[os.path.join(_ROOT, 'docker-compose.yml')])
    options = {
        "-d": True,
        "--no-color": True,
        "--quiet-pull": True,
        "--no-deps": True,
        "--force-recreate": True,
        "--always-recreate-deps": True,
        "--no-recreate": False,
        "--no-build": False,
        "--no-start": False,
        "--build": True,
        "--abort-on-container-exit": False,
        "--attach-dependencies": False,
        "--renew-anon-volumes": True,
        "--remove-orphans": True,
        "SERVICE": ["service", "worker", "redis", "monitor", "runner", "database"],
        "--scale": ["service=1"]
    }
    dc.command('up', options)


@cli.command()
def down():
    dc = Compose(files=[os.path.join(_ROOT, 'docker-compose.yml')])
    options = {
        "--rmi": "all",
        "--volumes": [],
        "--remove-orphans": True
    }
    dc.command('down', options)




# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
# pytype: skip-file

from __future__ import absolute_import

import json
import os
import time

import apache_beam as beam
import torch
from apache_beam.options.pipeline_options import PipelineOptions
from celery import Celery
from celery.utils.log import get_task_logger
from transformers import AutoTokenizer, AutoModelForTokenClassification

# loads the pretrained model which is then used for predicting values
# can read more about it here: https://github.com/huggingface/transformers
from langauge.core.dispatcher.options.pipeline_options import get_pipeline_options
from langauge.core.dispatcher.model.utils.download_azure import AzureBlobFileDownloader

logger = get_task_logger(__name__)

env = os.environ
CELERY_BROKER_URL = env.get("CELERY_BROKER_URL", "redis://localhost:6379/0")
CELERY_RESULT_BACKEND = env.get("CELERY_RESULT_BACKEND", "mongodb://root:root@localhost:27017/celery")

celery = Celery("task",
                broker=CELERY_BROKER_URL,
                backend=CELERY_RESULT_BACKEND)



class Tokenize(beam.DoFn):
    """
    The tokenizer is responsible for all the preprocessing the pretrained model expects, 
    and can be called directly on one (or list) of texts. 
    It outputs a dictionary which is then passed to the model for inference. 
    """

    def __init__(self, model_name):
        self.tokenizer = None
        self.modelName = model_name

    def setup(self):
        self.tokenizer = AutoTokenizer.from_pretrained(self.modelName)

    def process(self, sequence):
        input_ids = torch.tensor(self.tokenizer.encode(sequence)).unsqueeze(0)
        yield {'tokens': self.tokenizer.tokenize(sequence), 'inputs': input_ids}


class Predict(beam.DoFn):
    """
    Loads the pretrained model which is then used for predicting values
    Can read more about it here: https://github.com/huggingface/transformers
    """

    def __init__(self, modelName):
        self.model = None
        self.modelName = modelName

    def setup(self):
        self.model = AutoModelForTokenClassification.from_pretrained(self.modelName)

    def process(self, inputs):
        with torch.no_grad():
            outputs = self.model(inputs['inputs'])
        predictions = outputs[0].argmax(axis=-1)[0][1:-1]
        for token, pred in zip(inputs['tokens'], predictions):
            yield {'token': token, 'prediction': self.model.config.id2label[pred.numpy().item()]}



@celery.task(name='pubMedBERT', bind=True)
def pubmed_run(self, model_name, source, output_loc):
    """
    Task registered with celery.
    Creates & submits a beam pipeline to the provided flink cluster

    :param model_name: the model identifier
    :param source: location of input file
    :param output_loc: location of output file
    """
    logger.info("pubMedBERT started")
    start_time_of_task = time.time()
    sink = "/".join([output_loc, self.request.id.__str__()])
    self.update_state(state='PROGRESS', meta={'task': 'ner', 'model': model_name})

    beam_options = PipelineOptions(get_pipeline_options(model_name))
    if not os.path.exists('/models/' + model_name):
        # Initialize class and upload files
        azure_blob_file_downloader = AzureBlobFileDownloader()
        azure_blob_file_downloader.download_blobs_in_container(model_name + '.tar.xz')
    # time.sleep(120)
    with beam.Pipeline(options=beam_options) as p:
        main_pipeline = (p
                         # | 'Feature extraction' >> feature_extraction
                         | 'Read file' >> beam.io.ReadFromText(source)
                         | 'Tokenize' >> beam.ParDo(Tokenize('/models/' + model_name))
                         | 'Predict' >> beam.ParDo(Predict('/models/' + model_name))
                         # | "print" >> beam.Map(print)
                         # | 'Format as JSON' >> task.Map(json.dumps)
                         | 'Write predictions' >> beam.io.WriteToText(sink)
                         )
    time_taken = time.time() - start_time_of_task
    return {'task': 'ner', 'model': model_name, 'time': time_taken}


@celery.task(name='pubMedBERT.preview', bind=True)
def pubmed_preview(self, model_name, data, output_loc):
    """
    Task registered with celery.
    Creates & submits a beam pipeline to the provided flink cluster
    
    :param model_name: the model identifier
    :param data: list of string
    """
    logger.info("pubMedBERT preview started")
    sink = "/".join([output_loc, self.request.id.__str__()])
    beam_options = PipelineOptions(get_pipeline_options(model_name))
    self.update_state(state='PROGRESS', meta={'task': 'ner', 'model': model_name, 'preview': ''})
    if not os.path.exists('/models/' + model_name):
        # Initialize class and upload files
        azure_blob_file_downloader = AzureBlobFileDownloader()
        azure_blob_file_downloader.download_blobs_in_container(model_name + '.tar.xz')
    with beam.Pipeline(options=beam_options) as p:
        preview_pipeline = (p
                            | "Create" >> beam.Create(data)
                            | 'Strip' >> beam.Map(lambda text: text.strip('\n'))
                            | 'Tokenize' >> beam.ParDo(Tokenize('/models/' + model_name))
                            | 'Predict' >> beam.ParDo(Predict('/models/' + model_name))
                            | 'Combine' >> beam.combiners.ToList()
                            | 'Write predictions' >> beam.io.WriteToText(sink)
                            )

    for i in os.listdir(output_loc):
        if os.path.isfile(os.path.join(output_loc, i)) and self.request.id.__str__() in i:
            file_name = os.path.join(output_loc, i)
            with open(file_name, 'r') as file:
                data = file.read().replace("\'", "\"")
                return {'task': 'ner', 'model': model_name, 'preview': json.dumps(json.loads(data))}

    return {'task': 'ner', 'model': model_name, 'preview': json.dumps([])}



# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import json
import logging
import os

from flask import request
from flask_restplus import Resource, Namespace

from langauge.core.service.main.database import mongo

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

env = os.environ
"""
This namespace helps get all the supported/available tasks and models from the database. 
"""
config_ns = Namespace('config', description='All configuration settings')


@config_ns.route('/saveConfig')
class SaveConfig(Resource):
    @config_ns.doc('Used to save the selected options in a json format locally')
    def post(self):
        tasks = request.form.get('task')
        models = request.form.get('model')
        if not os.path.isdir(env.get('CONFIG_FOLDER')):
            os.makedirs(env.get('CONFIG_FOLDER'))
        with open(os.path.join(env.get('CONFIG_FOLDER'), 'task.json'), 'w+') as outfile:
            json.dump(tasks, outfile)
        with open(os.path.join(env.get('CONFIG_FOLDER'), 'models.json'), 'w+') as outfile:
            json.dump(models, outfile)
        return json.dumps({'success': True}), 200, {'ContentType': 'application/json'}



@config_ns.route('/runConfig')  # this is a job for GET, not POST
class RunConfig(Resource):
    @config_ns.doc('Used to retrieve the saved options from a json file locally')
    def post(self):
        data = {'task': {}, 'model': {}}
        config_dir = env.get('CONFIG_FOLDER')
        if os.path.isfile(config_dir + '/task.json'):
            with open(os.path.join(config_dir, 'task.json'), 'r+') as outfile:
                data['task'] = json.load(outfile)
            with open(os.path.join(config_dir, 'models.json'), 'r+') as outfile:
                data['model'] = json.load(outfile)
        return data, 200, {'ContentType': 'application/json'}



@config_ns.route('/models/<task>')
@config_ns.param('task', 'The task identifier')
class Models(Resource):
    @config_ns.doc('Returns all models supported for the selected task')
    def get(self, task):
        from bson.json_util import dumps
        documents = mongo.db.model.find_one({"task": task})
        return dumps(documents)



@config_ns.route('/task')
class Tasks(Resource):
    @config_ns.doc('Returns all the supported tasks')
    def get(self):
        from bson.json_util import dumps
        documents = mongo.db.task.find()
        return dumps(documents)




# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import os
import tarfile

from azure.storage.blob import BlobServiceClient

# IMPORTANT: Replace connection string with your storage account connection string
# Usually starts with DefaultEndpointsProtocol=https;...
MY_CONNECTION_STRING = os.environ.get("MY_CONNECTION_STRING")

# Replace with blob container
MY_BLOB_CONTAINER = "ner"

# Replace with the local folder where you want files to be downloaded
LOCAL_BLOB_PATH = "/models"


class AzureBlobFileDownloader:
    def __init__(self):
        print("Intializing AzureBlobFileDownloader")
        # Initialize the connection to Azure storage account
        self.blob_service_client = BlobServiceClient(account_url="https://langauge.blob.core.windows.net")
        self.my_container = self.blob_service_client.get_container_client(MY_BLOB_CONTAINER)

    def save_and_extract_blob(self, file_name, file_content):
        # Get full path to the file
        download_file_path = os.path.join(LOCAL_BLOB_PATH, file_name)
        # for nested blobs, create local path as well!
        os.makedirs(os.path.dirname(download_file_path), exist_ok=True)
        with open(download_file_path, "wb") as file:
            file_content.readinto(file)
        tar = tarfile.open(download_file_path)
        tar.extractall(LOCAL_BLOB_PATH)
        tar.close()
        os.remove(download_file_path)

    def download_blobs_in_container(self, file_name):
        my_blobs = self.my_container.list_blobs(name_starts_with=file_name)
        for blob in my_blobs:
            blob_data = self.my_container.get_blob_client(blob).download_blob()
            self.save_and_extract_blob(blob.name, blob_data)



# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from __future__ import absolute_import

import json
import logging
import os
from itertools import islice

from flask import request, url_for
from langauge.core.service.main.worker import celery
from langauge.core.service.endpoints.job_status import task_already_running
from werkzeug.utils import secure_filename
from flask_restplus import Resource, Namespace

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

env = os.environ

"""
Exposes two endpoints, one for the actual run and another one for the preview. 
Takes in common inputs: model and the channel information. 
"""
ner_ns = Namespace('ner',
                   description='All requests related to Named Entity Recognition Task')



@ner_ns.route('/<model>/<channel>')
@ner_ns.param('model', 'The model identifier')
@ner_ns.param('channel', 'The channel identifier')
class ModelRun(Resource):
    @ner_ns.doc('Create & submits an async inference task')
    def post(self, model, channel):
        logger.info("in /ner/ModelRun")
        try:
            if not (int(channel) == 0 or int(channel) == 1):
                channel = '0'
        except ValueError:
            channel = '0'

        file = request.files['file']

        # input file setup
        target = os.path.join(env.get('UPLOAD_FOLDER'), 'ner', model)
        if not os.path.isdir(target):
            os.makedirs(target)
        filename = secure_filename(file.filename)
        input_dest = "/".join([target, filename])
        file.save(input_dest)

        # output file setup
        output = os.path.join(env.get('OUTPUT_FOLDER'))
        if not os.path.isdir(output):
            os.makedirs(output)

        if not task_already_running(str(channel)):
            # call async task
            logger.info("starting async task")
            task = celery.send_task(model.split("_")[0],
                                    args=[model, input_dest, output])
            return url_for('task_status', task_id=task.id), 202, {}
        else:
            logger.info('another task already running')
            return "wait for the task to complete", 425, {}


@ner_ns.route('/preview/<model>/<num_lines>/<channel>')
@ner_ns.param('model', 'The model identifier')
@ner_ns.param('num_lines', 'initial number of lines to be executed for the preview')
@ner_ns.param('channel', 'The channel identifier')
class ModelPreview(Resource):
    @ner_ns.doc('Create & submits an async inference preview')
    def post(self, model, num_lines, channel):
        logger.info("in /preview/ner/ModelPreview")
        try:
            if not (int(channel) == 0 or int(channel) == 1):
                channel = '0'
        except ValueError:
            channel = '0'
        try:
            num_lines = int(num_lines)
        except ValueError:
            num_lines = 5

        file = request.files['file']

        target = os.path.join(env.get('UPLOAD_FOLDER'), 'ner', model)
        if not os.path.isdir(target):
            os.makedirs(target)
        filename = secure_filename(file.filename)
        input_dest = "/".join([target, filename])
        file.save(input_dest)
        with open(input_dest, 'r') as infile:
            head = list(islice(infile, num_lines))

        # output file setup
        output = os.path.join(env.get('OUTPUT_FOLDER'))
        if not os.path.isdir(output):
            os.makedirs(output)

        if not task_already_running(str(channel)):
            task = celery.send_task(model.split("_")[0]+'.preview',
                                    args=[model, head, output])
            return url_for('task_status', task_id=task.id), 202, {}
        else:
            return "wait for the task to complete", 425, {}




#!/bin/env python3
# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from langauge.core.service import main
from flask_socketio import SocketIO

app = main.create_app(debug=True)
socketio = SocketIO(app)

host = app.config.get("HOST")
port = app.config.get("PORT")

if __name__ == '__main__':
    print("Server running at http://%s:%s" % (host, port))
    socketio.run(app, host, port=int(port))



# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from flask import Flask
from flask_restplus import Api

from langauge.core.service.main.database import mongo
from flask_cors import CORS
from flask_socketio import SocketIO

socketio = SocketIO()


def create_app(debug=True):
    """
    Create an application.
    
    :param debug: whether to run Flask in debug Mode.
    :type debug: boolean
    """
    app = Flask(__name__)
    CORS(app, resources={r"/*": {"origins": "*"}})
    app.debug = debug
    app.config.from_object('configs')

    from langauge.core.service.endpoints.job_status import celery_task_ns
    from langauge.core.service.endpoints.ner import ner_ns
    from langauge.core.service.endpoints.config import config_ns
    # blueprint = Blueprint('api', __name__)
    api = Api(title='LanGauge API', version='1.0')
    api.add_namespace(celery_task_ns)
    api.add_namespace(ner_ns)
    api.add_namespace(config_ns)
    # main.register_blueprint(blueprint)
    api.init_app(app)

    socketio.init_app(app)

    mongo.init_app(app)

    return app



# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from flask_pymongo import PyMongo
mongo = PyMongo()


import os
from setuptools import setup, find_packages


def package_files(directory):
    paths = []
    for (path, _, filenames) in os.walk(directory):
        for filename in filenames:
            name, file_extension = os.path.splitext(filename)
            if file_extension == '.txt' or file_extension == '.yml' \
                    or filename == '.env' or name == 'Dockerfile':
                paths.append(os.path.join('..', path, filename))
    return paths


def package_all_files(directory):
    paths = []
    for (path, _, filenames) in os.walk(directory):
        for filename in filenames:
            if os.path.splitext(filename)[1] != '.local':
                paths.append(os.path.join('..', path, filename))
    return paths


all_files = package_files("langauge/")
ui = package_all_files("langauge/ui/")
db = package_all_files("langauge/core/database/init-db.d")


setup(
    name='langauge',
    version="0.0.3",
    packages=find_packages(exclude=['tests', 'tests.*']),
    package_data={"langauge": all_files+ui+db},
    license='MIT',
    description="LanGauge: 'gauge' the usefulness of open-source medical datasets",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    author='Flapmax',
    url='https://github.com/flapmx/LanGauge-INTERNAL',
    python_requires='>=3.6',
    keywords=['AI', 'NLP', 'Flapmax'],
    install_requires=[
        'click',
        'rapydo-controller'
    ],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
    entry_points={
        "console_scripts": [
            "langauge=langauge.cli:cli"
        ]
    }
)


# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

"""
Definitions of click options shared by several CLI commands.
"""
import click


HOST = click.option("--host", "-h", default="127.0.0.1",
                    help="The network address to listen on (nginx.conf: 127.0.0.1). "
                         "Use 0.0.0.0 to bind to all addresses if you want to access the tracking "
                         "server from other machines.")

PORT = click.option("--port", "-p", default="8080",
                    help="The port to listen on (nginx.conf: 8080).")

BACKEND = click.option("--backend", "-b", default="127.0.0.1:5000",
                    help="The backend address & port to listen on (nginx.conf: 127.0.0.1:5000). ")

BACKENDPORT = click.option("--backendport", "-bp", default="5000",
                    help="The REST backend port to listen on (nginx.conf: 5000).")


# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
# pytype: skip-file

from __future__ import absolute_import

import argparse
import logging

import apache_beam as beam
import torch
from apache_beam.options.pipeline_options import PipelineOptions
from apache_beam.options.pipeline_options import SetupOptions
from transformers import AutoTokenizer, AutoModelForTokenClassification


class Tokenize(beam.DoFn):

    def __init__(self, modelName):
        self.tokenizer = None
        self.modelName = modelName

    def setup(self):
        self.tokenizer = AutoTokenizer.from_pretrained(self.modelName)

    def process(self, sequence):
        input_ids = torch.tensor(self.tokenizer.encode(sequence)).unsqueeze(0)
        yield {'tokens': self.tokenizer.tokenize(sequence), 'inputs': input_ids}
        # tokens = self.tokenizer.tokenize(sequence)
        # inputs = self.tokenizer.encode(sequence, return_tensors="pt")
        # yield {'tokens': tokens, 'inputs': inputs}


# loads the pretrained model which is then used for predicting values
# can read more about it here: https://github.com/huggingface/transformers
class Predict(beam.DoFn):
    def __init__(self, modelName):
        self.model = None
        self.modelName = modelName

    def setup(self):
        self.model = AutoModelForTokenClassification.from_pretrained(self.modelName)

    def process(self, inputs):
        with torch.no_grad():
            outputs = self.model(inputs['inputs'])
        predictions = outputs[0].argmax(axis=-1)[0][1:-1]
        for token, pred in zip(inputs['tokens'], predictions):
            yield {'token': token, 'prediction': self.model.config.id2label[pred.numpy().item()]}
        # outputs = self.model(inputs['inputs'])[0]
        # predictions = torch.argmax(outputs, dim=2)
        # for token, prediction in zip(inputs['tokens'], predictions[0].numpy()):
        #     yield {'token': token, 'prediction': prediction}


def run(modelName, source, sink, beam_options=None):
    with beam.Pipeline(options=beam_options) as p:
        _ = (p
             | 'Read file' >> source
             | 'Split' >> beam.Map(lambda text: text.split('\t')[0])
             | 'Tokenize' >> beam.ParDo(Tokenize(modelName))
             | 'Predict' >> beam.ParDo(Predict(modelName))
             #  | "print" >> task.Map(print)
             # | 'Format as JSON' >> task.Map(json.dumps)
             | 'Write predictions' >> sink
             )


if __name__ == '__main__':
    logging.getLogger().setLevel(logging.INFO)

    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument(
        '--input',
        dest='input',
        # nginx.conf='/home/subham/PycharmProjects/LanGauge-INTERNAL/data/dev/two.txt',
        default='/home/subham/PycharmProjects/LanGauge-INTERNAL/data/dev/one.txt',
        help='Input file to process.')
    parser.add_argument(
        '--output',
        dest='output',
        default='/home/subham/PycharmProjects/LanGauge-INTERNAL/out.txt',
        help='Output file to write results to.')
    parser.add_argument(
        '--modelName',
        dest='modelName',
        # nginx.conf='allenai/scibert_scivocab_uncased',
        # nginx.conf='microsoft/BiomedNLP-PubMedBERT-base-uncased-abstract',
        # nginx.conf='fran-martinez/scibert_scivocab_cased_ner_jnlpba',
        default='/home/subham/Downloads/ner_trained_models/pubMedBERT_model_BC5CDR_chem',
        help='Model Name for prediction')
    args, pipeline_args = parser.parse_known_args()

    pipeline_args.extend([
        "--runner=PortableRunner",
        "--job_endpoint=localhost:8099",
        # "--runner=FlinkRunner",
        # "--flink_master=localhost:8081",
        # User code is executed within the same process that submitted the dispatcher.
        # This option is useful for local testing. However, it is not suitable for a
        # production environment, as it performs work on the machine the job originated from.
        "--environment_type=LOOPBACK"
        # "--environment_type=DOCKER"
    ])
    print(pipeline_args)
    beam_options = PipelineOptions(pipeline_args)
    beam_options.view_as(SetupOptions).save_main_session = True
    source = beam.io.ReadFromText(args.input)
    sink = beam.io.WriteToText(args.output)
    # [START predict_call_run]
    run(
        args.modelName,
        source,
        sink,
        beam_options)
    # [END predict_call_run]



# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import os

from celery import Celery

env = os.environ
CELERY_BROKER_URL = env.get('CELERY_BROKER_URL', 'redis://localhost:6379')
CELERY_RESULT_BACKEND = env.get("CELERY_RESULT_BACKEND", "mongodb://root:root@localhost:27017/celery")

celery = Celery('task',
                broker=CELERY_BROKER_URL,
                backend=CELERY_RESULT_BACKEND)



# Copyright (c) FlapMX LLC.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

def get_pipeline_options(job_name, runner='PortableRunner', max_num_workers=1):
    """Build dispatcher options for profile_api."""
    project = 'LanGauge-INTERNAL'
    pipe_options = [
        '--project=' + project,
        '--runner=' + runner,
        "--job_endpoint=localhost:8099",
        '--environment_type=LOOPBACK',
        '--job_name=' + '%s-%s' % (project, job_name),
        '--max_num_workers=' + str(max_num_workers)
    ]
    return pipe_options